#include "Structure/CharData.h"
#include "Structure/Dictionary.h"
#include "Structure/Tag.h"

#include <iostream>
#include <string>

#include <qfontmetrics.h>
#include <qregexp.h>

CharData::CharData(const QString& aContent)
	: Element(), Content(aContent), Translation("")
{
}

CharData::CharData(const CharData& anOther)
	: Element(anOther), Content(anOther.Content)
{
}

CharData::~CharData()
{
}

CharData& CharData::operator=(const CharData& anOther)
{
	if (this != &anOther)
	{
		Element::operator=(*this);
		Content = anOther.Content;
	}
	return *this;
}

CharData* CharData::copy() const
{
	return new CharData(*this);
}

unsigned int CharData::updateHeightToWidth(const QFontMetrics& M, unsigned int Width, unsigned int aStart)
{
	wrapWords(M, Width, Content, AsLines);
	wrapWords(M, Width, Translation, TranslationAsLines);
	Height = AsLines.size();
	if (Height < TranslationAsLines.size())
		Height = TranslationAsLines.size();
	Start = aStart;
	return Start+Height;
}

QString CharData::contentFromLine(unsigned int aLine, unsigned int Translation)
{
	if ( (Translation == 0) && (aLine >= Start) && (aLine < (Start+AsLines.size())) )
		return AsLines[aLine-Start];
	else if ( (Translation == 1) && (aLine >= Start) && (aLine < (Start+TranslationAsLines.size())) )
		return TranslationAsLines[aLine-Start];
	return "";
}

Element* CharData::elementFromLine(unsigned int aLine)
{
	if ( (aLine >= Start) && (aLine < (Start+Height)) )
		return this;
	return 0;
}


void CharData::wrapWords(const QFontMetrics& M, unsigned int aWidth, 
		const QString& Source, std::vector<QString>& Target)
{
	Target.clear();
	QString ToDo(Source);
	while (!ToDo.isEmpty())
	{
		int pos = ToDo.find(' ',1);
		if (pos == -1)
			pos = ToDo.length();
		QString Line(ToDo.left(pos));
		ToDo.remove(0,pos);
		while (!ToDo.isEmpty())
		{
			pos = ToDo.find(' ',1);
			if (pos == -1)
				pos = ToDo.length();
			QString Word(ToDo.left(pos));
			if (M.width(Line+Word) > static_cast<int>(aWidth))
				break;
			Line += Word;
			ToDo.remove(0,pos);
		}
		Target.push_back(Line);
		Line = "";
		if (ToDo.find(' ') == 0)
			ToDo.remove(0,1);
	}
}

QString CharData::original() const
{
	return Content;
}

QString CharData::translation() const
{
	return Translation;
}

void CharData::setTranslation(const QString& aText)
{
	Translation = aText;
}

void CharData::acceptTranslation(bool anAccept)
{
	TranslationAccepted = true;
	if (Parent)
		Parent->updateTranslationAccepted();
}

Element* CharData::next()
{
	if (Parent)
		return Parent->next(this);
	return 0;
}

Element* CharData::first()
{
	return this;
}

void CharData::save(std::ostream& File, const QString& Ident)
{
	for (unsigned int i=0; i<TranslationAsLines.size(); ++i)
	{
		std::string Out;
		for (unsigned int j=0; j<TranslationAsLines[i].length(); ++j)
		{
			char c = TranslationAsLines[i][j].latin1();
			if ( (c == 0x9) || (c == 0xa) || (c == 0xd) || (c>=0x20) )
				Out += c;
			else
			{
				unsigned short s = TranslationAsLines[i][j].unicode();
				Out += "&#x";
				Out += QString::number(s,16).latin1();
				Out += ";";
			}
		}
		File << (const char*)Ident << Out << std::endl;
	}
}

void CharData::use(const Dictionary& aDict)
{
	if (!TranslationAccepted)
	{
		if (aDict.contains(Content))
		{
			setTranslation(aDict.translate(Content));
			acceptTranslation(true);
		}
	}
}
