#include "Structure/Dictionary.h"

#include <fstream>

void Dictionary::clear()
{
	Content.clear();
}

void Dictionary::add(const QString& Original, const QString& Translation)
{
	Content[Original] = Translation;
}

void Dictionary::save(const QString& aName)
{
	std::ofstream of(aName);
	of << "<?xml version='1.0' encoding='utf-8'?>" << std::endl
		<< "<dictionary>" << std::endl;
	for (std::map<QString,QString>::iterator i=Content.begin(); i!=Content.end(); ++i)
	{
		of << "<message>" << std::endl
			<< " <original>" << std::endl
			<< "  " << toXML(i->first) << std::endl
			<< " </original>" << std::endl
			<< " <translation>" << std::endl
			<< "  " << toXML(i->second) << std::endl
			<< " </translation>" << std::endl
			<< "</message>" << std::endl;
	}
	of << "</dictionary>" << std::endl;
}

std::string Dictionary::toXML(const QString& aString)
{
	std::string Out;
	for (unsigned int j=0; j<aString.length(); ++j)
	{
		char c = aString[j].latin1();
		if ( (c == 0x9) || (c == 0xa) || (c == 0xd) || (c>=0x20) )
			Out += c;
		else
		{
			unsigned short s = aString[j].unicode();
			Out += "&#x";
			Out += QString::number(s,16).latin1();
			Out += ";";
		}
	}
	return Out;
}

bool Dictionary::contains(const QString& aName) const
{
	return Content.find(aName) != Content.end();
}

QString Dictionary::translate(const QString& aName) const
{
	std::map<QString, QString>::const_iterator i = Content.find(aName);
	if (i != Content.end())
		return i->second;
	return "";
}
