#include "Structure/Element.h"
#include "Structure/Tag.h"

Element::Element()
	: Height(0), Start(0), Parent(0), TranslationAccepted(false)
{
}

Element::Element(const Element& anOther)
	: Height(anOther.Height), Start(anOther.Start), Parent(0),
	TranslationAccepted(anOther.TranslationAccepted)
{
}

Element::~Element()
{
}


Element& Element::operator=(const Element& anOther)
{
	if (this != &anOther)
	{
		Height = anOther.Height;
		Start = anOther.Start;
		TranslationAccepted = anOther.TranslationAccepted;
	}
	return *this;
}

unsigned int Element::height() const
{
	return Height;
}

unsigned int Element::start() const
{
	return Start;
}

unsigned int Element::stop() const
{
	return Start+Height;
}

void Element::setParent(Tag* aParent)
{
	Parent = aParent;
}

Tag* Element::parent()
{
	return Parent;
}

const Tag* Element::parent() const
{
	return Parent;
}

unsigned int Element::depth() const
{
	unsigned int R = 0;
	const Element* E = parent();
	while (E)
	{
		++R;
		E = E->parent();
	}
	return R;
}

bool Element::isTranslationAccepted() const
{
	return TranslationAccepted;
}
