#include "Tag.h"

Tag::Tag(const QString& aName)
	: Element(), Name(aName)
{
}

Tag::Tag(const Tag& anOther)
	: Element(anOther)
{
	for (unsigned int i=0; i<anOther.Children.size(); ++i)
		Children.push_back(anOther.Children[i]->copy());
}

Tag::~Tag()
{
	for (unsigned int i=0; i<Children.size(); ++i)
		delete Children[i];
}

Tag& Tag::operator=(const Tag& anOther)
{
	if (this != &anOther)
	{
		Element::operator=(*this);
		for (unsigned int i=0; i<Children.size(); ++i)
			delete Children[i];
		Children.clear();
		for (unsigned int i=0; i<anOther.Children.size(); ++i)
			Children.push_back(anOther.Children[i]->copy());
	}
	return *this;
}

Tag* Tag::copy() const
{
	return new Tag(*this);
}

const QString& Tag::name() const
{
	return Name;
}

void Tag::add(Element* aChild)
{
	Children.push_back(aChild);
	aChild->setParent(this);
}

unsigned int Tag::updateHeightToWidth(const QFontMetrics& M, unsigned int aWidth, unsigned int aStart)
{
	Start = aStart;
	unsigned int Run = Start+1;
	for (unsigned int i=0; i<Children.size(); ++i)
		Run = Children[i]->updateHeightToWidth(M, aWidth-20, Run);
	Height = 2;
	for (unsigned int i=0; i<Children.size(); ++i)
		Height += Children[i]->height();
	return Start+Height;
}

QString Tag::contentFromLine(unsigned int aLine, unsigned int Translation)
{
	if (aLine == Start)
	{
//		if (Translation == 0)
			return "<"+Name+">";
	}
	else if (aLine == (Start+Height-1))
	{
//		if (Translation == 0)
			return "</"+Name+">";
	}
	else
		for (unsigned int i=0; i<Children.size(); ++i)
			if ((aLine >= Children[i]->start()) && (aLine < Children[i]->stop()))
				return Children[i]->contentFromLine(aLine, Translation);
	return "";
}

Element* Tag::elementFromLine(unsigned int aLine)
{
	if (aLine == Start)
		return this;
	else if (aLine == (Start+Height-1))
		return this;
	else
		for (unsigned int i=0; i<Children.size(); ++i)
			if ((aLine >= Children[i]->start()) && (aLine < Children[i]->stop()))
				return Children[i]->elementFromLine(aLine);
	return 0;
}

void Tag::updateTranslationAccepted()
{
	bool NewAccept = true;
	for (unsigned int i=0; i<Children.size(); ++i)
		NewAccept = NewAccept && Children[i]->isTranslationAccepted();
	if (NewAccept != TranslationAccepted)
	{
		TranslationAccepted = NewAccept;
		if (parent())
			parent()->updateTranslationAccepted();
	}
}

Element* Tag::next()
{
	if (parent())
		return parent()->next(this);
	return 0;
}

Element* Tag::next(Element* From)
{
	for (unsigned int i=1; i<Children.size(); ++i)
		if (Children[i-1] == From)
			return Children[i]->first();
	return next();
}

Element* Tag::first()
{
	if (Children.size())
		return Children[0]->first();
	return this;
}

void Tag::save(std::ostream& File, const QString& Ident)
{
	File << (const char*)Ident << "<" << (const char*)Name << ">" << std::endl;
	QString Ident2(Ident);
	Ident2 += "";
	for (unsigned int i=0; i<Children.size(); ++i)
		Children[i]->save(File,Ident2);
	File << (const char*)Ident << "</" << (const char*)Name << ">" << std::endl;
}

void Tag::use(const Dictionary& aDict)
{
	for (unsigned int i=0; i<Children.size(); ++i)
		Children[i]->use(aDict);
}
