#include "MainWindow.h"

#include <fstream>

#include <qaction.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qxml.h>

#include "IO/DocBookHandler.h"
#include "UI/TranslationText.h"
#include "UI/TranslationView.h"

MainWindow::MainWindow()
	: View(0), Text(0)
{
	setupUI();
}

void MainWindow::setupUI()
{
	// Views
	QWidget* W = new QWidget(this);
	setCentralWidget(W);
	View = new TranslationView(W,0);
	View->show();
	Text = new TranslationText(W);
	Text->show();
	Text->setEnabled(false);
	QLabel* L = new QLabel("Translation",W);
	L->setEnabled(false);
	QBoxLayout* MainLayout = new QVBoxLayout(W);
	MainLayout->addWidget(View);
	MainLayout->addWidget(L);
	MainLayout->addWidget(Text);

// Actions
	QAction* open = new QAction(tr("Open file"),tr("&Open"),CTRL+Key_O,this);
	connect(open, SIGNAL(activated()), this, SLOT(open()) );
	QAction* saveTranslation = new QAction(tr("Save translation"),tr("&Save"),
			CTRL+Key_S,this);
	connect(saveTranslation, SIGNAL(activated()), this, SLOT(save()));
	QAction* quit = new QAction(tr("Exit program"),tr("E&xit"),CTRL+Key_X,this);
	connect(quit, SIGNAL(activated()), this, SLOT(close()) );
	QAction* goodTranslation = new QAction(tr("Accept translation"),tr("&Accept"),
			CTRL+Key_A,this);
	connect(goodTranslation, SIGNAL(activated()), View, SLOT(acceptTranslation()));
	QAction* translationIsOriginal = new QAction(tr("Translation is original"),
			tr("From &Original"), CTRL+Key_O, this);
	connect(translationIsOriginal, SIGNAL(activated()), View, SLOT(translationIsOriginal()));
	QAction* saveDictionary = new QAction(tr("Save dictionary"),tr("&Save"),
			0, this);
	connect(saveDictionary, SIGNAL(activated()), this, SLOT(doSaveDictionary()));
	QAction* loadDictionary = new QAction(tr("Load dictionary"),tr("&Load"),
			0, this);
	connect(loadDictionary, SIGNAL(activated()), this, SLOT(doLoadDictionary()));

	// Menus
	QPopupMenu* file = new QPopupMenu(this);
	menuBar()->insertItem( tr("&File"), file );
	open->addTo(file);
	saveTranslation->addTo(file);
	quit->addTo(file);
	QPopupMenu* textMenu = new QPopupMenu(this);
	menuBar()->insertItem( tr("&Text"), textMenu);
	goodTranslation->addTo(textMenu);
	translationIsOriginal->addTo(textMenu);
	QPopupMenu* dictMenu = new QPopupMenu(this);
	menuBar()->insertItem( tr("&Dictionary"), dictMenu);
	loadDictionary->addTo(dictMenu);
	saveDictionary->addTo(dictMenu);

	// Connections
	connect(View,SIGNAL(charDataSelected(bool)), Text, SLOT(setEnabled(bool)));
	connect(View,SIGNAL(charDataSelected(bool)), L, SLOT(setEnabled(bool)));
	connect(View,SIGNAL(charDataSelected(bool)), goodTranslation, SLOT(setEnabled(bool)));
	connect(View,SIGNAL(charDataChanged(const QString&)), 
			Text, SLOT(setText(const QString&)));
	connect(Text,SIGNAL(textChangedTo(const QString&)),
			View, SLOT(setTranslationText(const QString&)));

}

void MainWindow::open()
{
	QString fileName = QFileDialog::getOpenFileName();
	if (fileName != QString::null)
	{
		DocBookHandler Parser;
		QFile xmlFile( fileName );
		QXmlInputSource source( &xmlFile );
		QXmlSimpleReader reader;
		reader.setContentHandler( &Parser );
		reader.parse(source);
		Parser.Book->updateHeightToWidth(View->fontMetrics(), 400 ,0);
		View->setRoot(Parser.Book);
		View->updateContents();
	}
}

void MainWindow::save()
{
	if (View->root())
	{
		QString fileName = QFileDialog::getSaveFileName();
		if (fileName)
		{
			std::ofstream File((const char*)fileName);
			View->root()->save(File,"");
		}
	}
}

void MainWindow::doSaveDictionary()
{
	QString s = QFileDialog::getSaveFileName();
	if (s != QString::null)
		View->saveDictionary(s);
}

void MainWindow::doLoadDictionary()
{
	QString s = QFileDialog::getSaveFileName();
	if (s != QString::null)
		View->loadDictionary(s);
}
